// Copyright 2014 The Macaron Authors
//
// Licensed under the Apache License, Version 2.0 (the "License"): you may
// not use this file except in compliance with the License. You may obtain
// a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

package toolbox

import (
	"os"
	"testing"
	"time"
)

func TestStatistic(t *testing.T) {
	tb := toolbox{
		UrlMap: &UrlMap{
			urlmap: make(map[string]map[string]*Statistics),
		},
	}

	tb.AddStatistics("POST", "/api/user", time.Duration(2000))
	tb.AddStatistics("POST", "/api/user", time.Duration(120000))
	tb.AddStatistics("GET", "/api/user", time.Duration(13000))
	tb.AddStatistics("POST", "/api/admin", time.Duration(14000))
	tb.AddStatistics("POST", "/api/user/unknwon", time.Duration(12000))
	tb.AddStatistics("DELETE", "/api/user", time.Duration(1400))
	tb.GetMap(os.Stdout)
	tb.JSON(os.Stdout)
}
